aboutgitcodebugslistschat
path: root/passt.h
diff options
context:
space:
mode:
authorStefano Brivio <sbrivio@redhat.com>2021-04-29 16:59:20 +0200
committerStefano Brivio <sbrivio@redhat.com>2021-04-29 17:15:26 +0200
commit605af213c5e0fa047f6d8caef5bcef61a0987c8d (patch)
tree45615e603964adee64bfecdc40e119bd33d77859 /passt.h
parent50bcddabc9e2c0dbd313a61cda1606045a67a8de (diff)
downloadpasst-605af213c5e0fa047f6d8caef5bcef61a0987c8d.tar
passt-605af213c5e0fa047f6d8caef5bcef61a0987c8d.tar.gz
passt-605af213c5e0fa047f6d8caef5bcef61a0987c8d.tar.bz2
passt-605af213c5e0fa047f6d8caef5bcef61a0987c8d.tar.lz
passt-605af213c5e0fa047f6d8caef5bcef61a0987c8d.tar.xz
passt-605af213c5e0fa047f6d8caef5bcef61a0987c8d.tar.zst
passt-605af213c5e0fa047f6d8caef5bcef61a0987c8d.zip
udp: Connection tracking for ephemeral, local ports, and related fixes
As we support UDP forwarding for packets that are sent to local ports, we actually need some kind of connection tracking for UDP. While at it, this commit introduces a number of vaguely related fixes for issues observed while trying this out. In detail: - implement an explicit, albeit minimalistic, connection tracking for UDP, to allow usage of ephemeral ports by the guest and by the host at the same time, by binding them dynamically as needed, and to allow mapping address changes for packets with a loopback address as destination - set the guest MAC address whenever we receive a packet from tap instead of waiting for an ARP request, and set it to broadcast on start, otherwise DHCPv6 might not work if all DHCPv6 requests time out before the guest starts talking IPv4 - split context IPv6 address into address we assign, global or site address seen on tap, and link-local address seen on tap, and make sure we use the addresses we've seen as destination (link-local choice depends on source address). Similarly, for IPv4, split into address we assign and address we observe, and use the address we observe as destination - introduce a clock_gettime() syscall right after epoll_wait() wakes up, so that we can remove all the other ones and pass the current timestamp to tap and socket handlers -- this is additionally needed by UDP to time out bindings to ephemeral ports and mappings between loopback address and a local address - rename sock_l4_add() to sock_l4(), no semantic changes intended - include <arpa/inet.h> in passt.c before kernel headers so that we can use <netinet/in.h> macros to check IPv6 address types, and remove a duplicate <linux/ip.h> inclusion Signed-off-by: Stefano Brivio <sbrivio@redhat.com>
Diffstat (limited to 'passt.h')
-rw-r--r--passt.h35
1 files changed, 20 insertions, 15 deletions
diff --git a/passt.h b/passt.h
index 714e7e5..009cedb 100644
--- a/passt.h
+++ b/passt.h
@@ -20,20 +20,23 @@ struct tap_msg {
/**
* struct ctx - Execution context
* @epollfd: file descriptor for epoll instance
* @fd_unix: AF_UNIX socket for tap file descriptor
* @v4: Enable IPv4 transport
* @mac: Host MAC address
* @mac_guest: Guest MAC address
* @addr4: IPv4 address for external, routable interface
* @mask4: IPv4 netmask, network order
* @gw4: Default IPv4 gateway, network order
* @dns4: IPv4 DNS address, network order
* @v6: Enable IPv6 transport
* @addr6: IPv6 address for external, routable interface
* @gw6: Default IPv6 gateway
* @dns4: IPv6 DNS address
* @ifn: Name of routable interface
*/
struct ctx {
int epollfd;
@@ -43,13 +46,15 @@ struct ctx {
int v4;
uint32_t addr4;
+ uint32_t addr4_seen;
uint32_t mask4;
uint32_t gw4;
uint32_t dns4;
int v6;
struct in6_addr addr6;
- struct in6_addr addr6_guest;
+ struct in6_addr addr6_seen;
+ struct in6_addr addr6_ll_seen;
struct in6_addr gw6;
struct in6_addr dns6;
@@ -20,20 +20,23 @@ struct tap_msg {
/**
* struct ctx - Execution context
- * @epollfd: file descriptor for epoll instance
- * @fd_unix: AF_UNIX socket for tap file descriptor
- * @v4: Enable IPv4 transport
- * @mac: Host MAC address
- * @mac_guest: Guest MAC address
- * @addr4: IPv4 address for external, routable interface
- * @mask4: IPv4 netmask, network order
- * @gw4: Default IPv4 gateway, network order
- * @dns4: IPv4 DNS address, network order
- * @v6: Enable IPv6 transport
- * @addr6: IPv6 address for external, routable interface
- * @gw6: Default IPv6 gateway
- * @dns4: IPv6 DNS address
- * @ifn: Name of routable interface
+ * @epollfd: file descriptor for epoll instance
+ * @fd_unix: AF_UNIX socket for tap file descriptor
+ * @v4: Enable IPv4 transport
+ * @mac: Host MAC address
+ * @mac_guest: Guest MAC address
+ * @addr4: IPv4 address for external, routable interface
+ * @addr4_seen: Latest IPv4 address seen as source from tap
+ * @mask4: IPv4 netmask, network order
+ * @gw4: Default IPv4 gateway, network order
+ * @dns4: IPv4 DNS address, network order
+ * @v6: Enable IPv6 transport
+ * @addr6: IPv6 address for external, routable interface
+ * @addr6_seen: Latest IPv6 global/site address seen as source from tap
+ * @addr6_ll_seen: Latest IPv6 link-local address seen as source from tap
+ * @gw6: Default IPv6 gateway
+ * @dns4: IPv6 DNS address
+ * @ifn: Name of routable interface
*/
struct ctx {